(*
  Thread unit that parses RFC and generates a HTML
*)

unit html;

interface

uses Classes, SysUtils, Dialogs, StrUtils, PerlRegEx;

type
  RFCtoHTML = class(TThread)
   private
    function WithTOC : Boolean;
    function WithoutTOC : Boolean;
   protected
    procedure Execute; override;
end;

implementation
uses rfc_gen;

//===============================================
function RFCtoHTML.WithTOC : Boolean;
var RFCFileText, HTMLFileText : TextFile;
    PageCount, TopicCount, {SubtopicCount,} TempVarInt : integer;
    RFCLine : string;
    TOCTopicList{, TOCSubtopicList} : TStringList;
    HasBeenWritten : Boolean;
    RegularExpTopics, RegularExpDate, RegularExpPages : TPerlRegEx;
begin
//=======
 PageCount := 0;
 TopicCount := -1;
 //SubtopicCount := 0;
 HasBeenWritten := False;
//=============
{$REGION 'Assigning files and creating lists'}
 try
  Form1.StatusBar1.Panels[0].Text := 'Assigning and creating files';
  AssignFile(RFCFileText,Form1.Edit1.Text);
  Reset(RFCFileText);
  AssignFile(HTMLFileText,rfc_gen.FileName);
  Rewrite(HTMLFileText);
  RegularExpTopics := TPerlRegEx.Create;
  RegularExpPages := TPerlRegEx.Create;
  RegularExpDate := TPerlRegEx.Create;
  TOCTopicList := TStringList.Create;
  //TOCSubtopicList := TStringList.Create;
  TOCTopicList.Sorted := False;
  //TOCSubtopicList.Sorted := False;
  Form1.StatusBar1.Panels[0].Text := 'Reading TOC into memory';
  RegularExpTopics.RegEx := '^\s*([\d]+\.?)*[\d]+\.?[^.0-9]*';
   for TempVarInt := 0 to Form1.Memo1.Lines.Count-1 do
    begin
     //if AnsiStartsText('*',Form1.Memo1.Lines.Strings[TempVarInt]) then TOCTopicList.Add(Trim(AnsiReplaceText(Form1.Memo1.Lines.Strings[TempVarInt],'*','')))
     //else if Form1.Memo1.Lines.Strings[TempVarInt] <> '' then TOCSubtopicList.Add(Trim(Form1.Memo1.Lines.Strings[TempVarInt]));
     //if Form1.Memo1.Lines.Strings[TempVarInt] <> '' then TOCTopicList.Add(Form1.Memo1.Lines.Strings[TempVarInt]);
     if Form1.Memo1.Lines.Strings[TempVarInt] <> '' then RegularExpTopics.Subject := Form1.Memo1.Lines.Strings[TempVarInt];
      RegularExpTopics.Match;
      TOCTopicList.Add(TrimRight(RegularExpTopics.MatchedText));
    end;
 except
   Result := False;
   Form1.StatusBar1.Panels[0].Text := 'Something went wrong with Assigning and creating files :(';
 end;
{$ENDREGION 'Assigning files and creating lists'}
//=============
{$REGION 'HTML/CSS code generation'}
//=======
{$REGION 'Style and button generation'}
 Form1.StatusBar1.Panels[0].Text := 'Generating HTML/CSS code';
  Write(HTMLFileText,'<html>'+#13#10+
          '<style type="text/css">'+#13#10+
          ' body {'+#13#10+
          '   font-family: verdana;'+#13#10+
          '   color: #000000; background-color: #ffffff;'+#13#10+
          ' }'+#13#10+
          ' pre {'+#13#10+
          '   font-size: 16;'+#13#10+
          ' }'+#13#10+
          ' table {'+#13#10+
          '   border: 2px solid #ED569B;'+#13#10+
          ' }'+#13#10+
          ' .date {'+#13#10+
          '   color: #669B00;'+#13#10+
          '   font-weight: bold;'+#13#10+
          ' }'+#13#10+
          ' .toclinks a:link {'+#13#10+
          '   color: #990000;'+#13#10+
          '   text-decoration: none;'+#13#10+
          ' }'+#13#10+
          ' .toclinks a:visited {'+#13#10+
          '   color: #333333;'+#13#10+
          '   text-decoration: none;'+#13#10+
          ' }'+#13#10+
          ' .toclinks a:active {'+#13#10+
          '   color: #333333;'+#13#10+
          '   text-decoration: none;'+#13#10+
          '   font-weight: bold;'+#13#10+
          ' }'+#13#10+
          ' .toclinks a:hover {'+#13#10+
          '   text-decoration: underline;'+#13#10+
          ' }'+#13#10+
          ' .page {'+#13#10+
          '   font-weight: bold;'+#13#10+
          ' }'+#13#10+
          ' .toptoc {'+#13#10+
          '   background: #C55612;'+#13#10+
          '   color: white;'+#13#10+
          ' }'+#13#10+
          ' .toptoc a:link {'+#13#10+
          '   text-decoration: none;'+#13#10+
          '   color: white;'+#13#10+
          ' }'+#13#10+
          ' .toptoc a:visited {'+#13#10+
          '   text-decoration: none;'+#13#10+
          '   color: white;'+#13#10+
          ' }'+#13#10+
          ' .toptoc a:active {'+#13#10+
          '   text-decoration: none;'+#13#10+
          '   color: white;'+#13#10+
          ' }'+#13#10+
          ' .toctopic {'+#13#10+
          '   font-size: 25px;'+#13#10+
          '   font-weight: bold;'+#13#10+
          ' }'+#13#10+
          ' .tocsubtopic {'+#13#10+
          '   font-size: 18px;'+#13#10+
          '   font-weight: bold;'+#13#10+
          ' }'+#13#10+
          '</style>'+#13#10+
          '<title>RFC generated by "RFC->HTML" - http://newage.ql.lt/projects/rfchtml/</title>'+#13#10+
          '<body>'+#13#10+
          '<a name="top"></a>'+#13#10+
          'Page navigation:'+#13#10+
          '<table>'+#13#10+
          ' <tr>'+#13#10+
          '  <td>'+#13#10);
{$REGION 'Page navigation code'}
 Form1.StatusBar1.Panels[0].Text := 'Generating page navigation code';
 RegularExpPages.RegEx := '\[\w+\s+\d{1,}\]';
 repeat
  ReadLn(RFCFileText,RFCLine); // read the document line by line
  RegularExpPages.Subject := RFCLine;
   //if AnsiContainsText(RFCLine,'[Page ') then // if page numeration was found...
   if RegularExpPages.Match = True then
    begin
     Inc(PageCount);
     Write(HTMLFileText,'<a href="#page'+IntToStr(PageCount)+'"><input type="button" value="Page '+IntToStr(PageCount)+'"></a>'+#13#10);
    end;
 until EoF(RFCFileText);
{$ENDREGION 'Page navigation code'}
  Write(HTMLFileText,'  </td>'+#13#10+
          ' </tr>'+#13#10+
          '</table>'+#13#10+
          '<pre>'+#13#10);
{$ENDREGION 'Style and button generation'}
//=======
{$REGION 'TOC link generation'}
Form1.StatusBar1.Panels[0].Text := 'Generating TOC links';
 Reset(RFCFileText);  // start from the beginning
  repeat
   ReadLn(RFCFileText,RFCLine); // read the document line by line
    for TempVarInt := 0 to (TOCTopicList.Count-1) do  // compare all items in the list to the RFCLine
     //if TempVarInt <= TOCTopicList.Count-1 then  // if the number is not bigger then there are items in the list
     if AnsiContainsText(RFCLine,Trim(TOCTopicList[TempVarInt])) then // if the line contains a topic in the list
      begin
    //====
       if (TempVarInt = 0) then // if the topic is the first one then add the toc tag on top
         Write(HTMLFileText,'<a name="toc"></a>'+#13#10+
               '<span class="toclinks">'+#13#10);
    //====
       Inc(TopicCount);
       Write(HTMLFileText,'<a href="#'+IntToStr(TopicCount)+'">'+TOCTopicList[TempVarInt]+'</a>'+#13#10);
       HasBeenWritten := True; // use this variable to mark if the line is written. This way the code avoids double-lines
       Break;
       end;
{$REGION 'Shit code'}
       {//Break;
      //end;
    //for TempVarInt := 0 to (TOCSubtopicList.Count-1) do
    end
    else
     begin
    ReadLn(RFCFileText,RFCLine);
     if AnsiContainsText(RFCLine,TOCSubtopicList[TempVarInt]) then
      begin
       Inc(SubtopicCount);
       Inc(LinesWithTopicCount);
       Write(HTMLFileText,'          <a href="#'+IntToStr(TopicCount)+IntToStr(SubtopicCount)+'">'+TOCSubtopicList[TempVarInt]+'</a>'+#13#10);
      end;
    { else
      begin
       Write(HTMLFileText,RFCLine+#13#10);
       Break;
      end;}
{$ENDREGION 'Shit code'}
     case HasBeenWritten of
      False : begin
               Write(HTMLFileText,RFCLine+#13#10);
               HasBeenWritten := False;
              end;
     end;
  until TopicCount = TOCTopicList.Count-1; //EoF(RFCFileText);
   Write(HTMLFileText,'</span>');
{$ENDREGION 'TOC link generation'}
//=======
{$REGION 'Reading and parsing what''s left'}
PageCount := 0;
TopicCount := -1;
Form1.StatusBar1.Panels[0].Text := 'Marking the pages and topics';
RegularExpPages.RegEx := '\[\w+\s+\d{1,}\]';  // regex to find all lines with [X Y] ; X being a word and Y being a number
RegularExpDate.RegEx := '(\d{1,2}\s)?[a-zA-Z]+\s(\d{1,2}(,)?\s)?\d{4}$';
RegularExpTopics.RegEx := '^(\d+\.)*(\s)?((\.)?\d+(\.)?(\s)?)?\w+(\s\w+)*?$';
 repeat
  ReadLn(RFCFileText,RFCLine); // read the document line by line
{$REGION 'Pages'}
   //if AnsiContainsText(RFCLine,'[Page ') then // if page numeration was found...
   RegularExpPages.Subject := RFCLine; // line to search with regex
   if RegularExpPages.Match = True then  // if a match was found
    begin
     Inc(PageCount);
     Write(HTMLFileText,'<br />'+#13#10+
           '<span class="toptoc"><a href="#top"> Back to top </a></span> &nbsp <span class="toptoc"><a href="#toc"> Back to TOC </a></span>'+#13#10+
           '<hr />'+#13#10);
     //Write(HTMLFileText,AnsiReplaceText(RFCLine,'[Page '+IntToStr(PageCount)+']','<span class="page"><a name="page'+IntToStr(PageCount)+'">[Page '+IntToStr(PageCount)+']</a></span>')+#13#10);
     RegularExpPages.Replacement := '<span class="page"><a name="page'+IntToStr(PageCount)+'">[Page '+IntToStr(PageCount)+']</a></span>'+#13#10;
     RegularExpPages.Replace; // replace with this ^
     Write(HTMLFileText,RegularExpPages.Subject); // 'RegularExp.Replace' replaces the match in RegularExp.Subject
     HasBeenWritten := True;
    end
   else HasBeenWritten := False;
   //else //Write(HTMLFileText,RFCLine+#13#10);
{$ENDREGION 'Pages'}

{$REGION 'Date'}
 if (HasBeenWritten = False) then
  RegularExpDate.Subject := RFCLine;
   if RegularExpDate.Match = True then
    begin
     RegularExpDate.Replacement := '<span class="date">'+RegularExpDate.MatchedText+'</span>';
     RegularExpDate.Replace;
     Write(HTMLFileText,RegularExpDate.Subject);
     HasBeenWritten := True;
    end;
   //else HasBeenWritten := False;
{$ENDREGION 'Date'}

{$REGION 'Topics'}
 if (HasBeenWritten = False) then
    //for TempVarInt := 0 to TOCTopicList.Count-1 do
     //if AnsiContainsText(RFCLine,Trim(TOCTopicList[TempVarInt])){ or AnsiContainsStr(RFCLine,Trim(TOCTopicList[TempVarInt])+#13#10) or AnsiContainsStr(RFCLine,#13#10+Trim(TOCTopicList[TempVarInt])+#13#10) }then
      RegularExpTopics.Subject := RFCLine;
     if RegularExpTopics.Match = True then
      begin
       Inc(TopicCount);
       Write(HTMLFileText,'<span class="toctopic"><a name="'+IntToStr(TopicCount)+'">'+RegularExpTopics.MatchedText{TOCTopicList[TempVarInt]}+'</a></span>'+#13#10);
       HasBeenWritten := True;
       //Break;
      end;
    case HasBeenWritten of
     False : Write(HTMLFileText,RFCLine+#13#10);
    end;
{$ENDREGION 'Topics'}
 until EoF(RFCFileText);
{$ENDREGION 'Reading and parsing what''s left'}
//=======
Form1.StatusBar1.Panels[0].Text := 'Finishing up the code';
 Write(HTMLFileText,'</pre>'+#13#10+
                    '</body>'+#13#10+
                    '</html>');
{$ENDREGION 'HTML/CSS code generation'}
//=============
Form1.StatusBar1.Panels[0].Text := 'Closing files';
 try
  CloseFile(RFCFileText);
  CloseFile(HTMLFileText);
  TOCTopicList.Free;
 except
   Result := False;
   Form1.StatusBar1.Panels[0].Text := 'Something went wrong with Assigning and creating files :(';
 end;
//=============
 Result := True;
 RegularExpPages.Free;
 RegularExpTopics.Free;
end;
//===============================================
function RFCtoHTML.WithoutTOC : Boolean;
var RFCFileText, HTMLFileText : TextFile;
    PageCount, TopicCount, TempVarInt : integer;
    RFCLine : string;
    TOCTopicList : TStringList;
    HasBeenWritten : Boolean;
    RegularExp : TPerlRegEx;
    RegularExpDate : TPerlRegEx;
begin
//=============
{$REGION 'Assigning files and creating lists'}
 try
  Form1.StatusBar1.Panels[0].Text := 'Assigning and creating files';
  AssignFile(RFCFileText,Form1.Edit1.Text);
  Reset(RFCFileText);
  AssignFile(HTMLFileText,rfc_gen.FileName);
  Rewrite(HTMLFileText);
  RegularExp := TPerlRegEx.Create;
  RegularExpDate := TPerlRegEx.Create;
  TOCTopicList := TStringList.Create;
  TOCTopicList.Sorted := False;
 except
   Result := False;
   Form1.StatusBar1.Panels[0].Text := 'Something went wrong with Assigning and creating files :(';
 end;
{$ENDREGION 'Assigning files and creating lists'}
//=============
{$REGION 'Style, button and TOC generation'}
 Form1.StatusBar1.Panels[0].Text := 'Generating HTML/CSS code';
  Write(HTMLFileText,'<html>'+#13#10+
          '<style type="text/css">'+#13#10+
          ' body {'+#13#10+
          '   font-family: verdana;'+#13#10+
          '   color: #000000; background-color: #ffffff;'+#13#10+
          ' }'+#13#10+
          ' pre {'+#13#10+
          '   font-size: 16;'+#13#10+
          ' }'+#13#10+
          ' table {'+#13#10+
          '   border: 2px solid #ED569B;'+#13#10+
          ' }'+#13#10+
          ' .date {'+#13#10+
          '   color: #669B00;'+#13#10+
          '   font-weight: bold;'+#13#10+
          ' }'+#13#10+
          ' .toclinks a:link {'+#13#10+
          '   color: #990000;'+#13#10+
          '   text-decoration: none;'+#13#10+
          ' }'+#13#10+
          ' .toclinks a:visited {'+#13#10+
          '   color: #333333;'+#13#10+
          '   text-decoration: none;'+#13#10+
          ' }'+#13#10+
          ' .toclinks a:active {'+#13#10+
          '   color: #333333;'+#13#10+
          '   text-decoration: none;'+#13#10+
          '   font-weight: bold;'+#13#10+
          ' }'+#13#10+
          ' .toclinks a:hover {'+#13#10+
          '   text-decoration: underline;'+#13#10+
          ' }'+#13#10+
          ' .page {'+#13#10+
          '   font-weight: bold;'+#13#10+
          ' }'+#13#10+
          ' .toptoc {'+#13#10+
          '   background: #C55612;'+#13#10+
          '   color: white;'+#13#10+
          ' }'+#13#10+
          ' .toptoc a:link {'+#13#10+
          '   text-decoration: none;'+#13#10+
          '   color: white;'+#13#10+
          ' }'+#13#10+
          ' .toptoc a:visited {'+#13#10+
          '   text-decoration: none;'+#13#10+
          '   color: white;'+#13#10+
          ' }'+#13#10+
          ' .toptoc a:active {'+#13#10+
          '   text-decoration: none;'+#13#10+
          '   color: white;'+#13#10+
          ' }'+#13#10+
          ' .toctopic {'+#13#10+
          '   font-size: 25px;'+#13#10+
          '   font-weight: bold;'+#13#10+
          ' }'+#13#10+
          '</style>'+#13#10+
          '<title>RFC generated by "RFC->HTML" - http://newage.ql.lt/projects/rfchtml/</title>'+#13#10+
          '<body>'+#13#10+
          '<a name="top"></a>'+#13#10+
          'Page navigation:'+#13#10+
          '<table>'+#13#10+
          ' <tr>'+#13#10+
          '  <td>'+#13#10);
{$REGION 'Page navigation code'}
 Form1.StatusBar1.Panels[0].Text := 'Generating page navigation code';
 RegularExp.RegEx := '\[\w+\s+\d{1,}\]';
 PageCount := 0;
 repeat
  ReadLn(RFCFileText,RFCLine); // read the document line by line
  RegularExp.Subject := RFCLine;
   if RegularExp.Match = True then
    begin
     Inc(PageCount);
     Write(HTMLFileText,'<a href="#page'+IntToStr(PageCount)+'"><input type="button" value="Page '+IntToStr(PageCount)+'"></a>'+#13#10);
    end;
 until EoF(RFCFileText);
{$ENDREGION 'Page navigation code'}
  Write(HTMLFileText,'  </td>'+#13#10+
          ' </tr>'+#13#10+
          '</table>'+#13#10+
          '<pre>'+#13#10+
          '<hr />'+#13#10);
{$REGION 'TOC navigation code'}
  Form1.StatusBar1.Panels[0].Text := 'Generating TOC navigation code';
  Reset(RFCFileText);
  RegularExp.RegEx := '^(\d+\.)*(\s)?((\.)?\d+(\.)?(\s)?)?\w+(\s\w+)*?$'; // regex to find all topics
  RegularExp.Options := [preCaseLess]; // make the search case insensitive
  Reset(RFCFileText);
  TopicCount := -1;
 repeat
  ReadLn(RFCFileText,RFCLine);
   RegularExp.Subject := RFCLine;
   if (RegularExp.Match = True) then
    begin
     TOCTopicList.Add(RegularExp.MatchedText);
     Inc(TopicCount);
      if TopicCount = 0 then
       begin
        Write(HTMLFileText,'<a name="toc"></a>'+#13#10+
              '<span class="toclinks">'+#13#10);
       end;
     Write(HTMLFileText,'<a href="#'+IntToStr(TopicCount)+'">'+RegularExp.MatchedText+'</a>'+#13#10);
    end;
 until EoF(RFCFileText);
 Write(HTMLFileText,'</span>'+#13#10+
       '<hr />'+#13#10);
{$ENDREGION 'TOC navigation code'}
{$ENDREGION 'Style, button and TOC generation'}
//=============
{$REGION 'Reading and parsing what''s left'}
PageCount := 0;
TopicCount := -1;
Form1.StatusBar1.Panels[0].Text := 'Searching for topics and creating the TOC';
RegularExp.RegEx := '\[\w+\s+\d{1,}\]';  // regex to find all lines with [X Y] ; X being a word and Y being a number
RegularExpDate.RegEx := '(\d{1,2}\s)?[a-zA-Z]+\s(\d{1,2}(,)?\s)?\d{4}$';
Reset(RFCFileText);
 repeat
  ReadLn(RFCFileText,RFCLine); // read the document line by line
{$REGION 'Pages'}
   RegularExp.Subject := RFCLine; // line to search with regex
   if RegularExp.Match = True then  // if a match was found
    begin
     Inc(PageCount);
     Write(HTMLFileText,'<br />'+#13#10+
           '<span class="toptoc"><a href="#top"> Back to top </a></span> &nbsp <span class="toptoc"><a href="#toc"> Back to TOC </a></span>'+#13#10+
           '<hr />'+#13#10);
     RegularExp.Replacement := '<span class="page"><a name="page'+IntToStr(PageCount)+'">[Page '+IntToStr(PageCount)+']</a></span>'+#13#10;
     RegularExp.Replace; // replace with this ^
     Write(HTMLFileText,RegularExp.Subject); // 'RegularExp.Replace' replaces the match in RegularExp.Subject
     HasBeenWritten := True;
    end
   else HasBeenWritten := False;
{$ENDREGION 'Pages'}

{$REGION 'Date'}
 if (HasBeenWritten = False) then
  RegularExpDate.Subject := RFCLine;
   if RegularExpDate.Match = True then
    begin
     RegularExpDate.Replacement := '<span class="date">'+RegularExpDate.MatchedText+'</span>';
     RegularExpDate.Replace;
     Write(HTMLFileText,RegularExpDate.Subject);
     HasBeenWritten := True;
    end;
{$ENDREGION 'Date'}

{$REGION 'Topics'}
 if (HasBeenWritten = False) then
    for TempVarInt := 0 to TOCTopicList.Count-1 do
     if AnsiContainsStr(RFCLine,Trim(TOCTopicList[TempVarInt])) then
      begin
       Inc(TopicCount);
       Write(HTMLFileText,'<span class="toctopic"><a name="'+IntToStr(TopicCount)+'">'+TOCTopicList[TempVarInt]+'</a></span>'+#13#10);
       HasBeenWritten := True;
       Break;
      end;
{$ENDREGION 'Topics'}
    case HasBeenWritten of
     False : Write(HTMLFileText,RFCLine+#13#10);
    end;
 until EoF(RFCFileText);
{$ENDREGION 'Reading and parsing what''s left'}
//=======
Form1.StatusBar1.Panels[0].Text := 'Finishing up the code';
 Write(HTMLFileText,'</pre>'+#13#10+
                    '</body>'+#13#10+
                    '</html>');
{$ENDREGION 'HTML/CSS code generation'}
//=============
Form1.StatusBar1.Panels[0].Text := 'Closing files';
 try
  CloseFile(RFCFileText);
  CloseFile(HTMLFileText);
 except
   Result := False;
   Form1.StatusBar1.Panels[0].Text := 'Something went wrong with closing files :(';
 end;
//=============
 Result := True;
 RegularExp.Free;
//=============
end;
//===============================================
procedure RFCtoHTML.Execute;
var EverythingOK : Boolean;
begin
FreeOnTerminate := True;
 if (Form1.CheckBox1.Checked = True) then EverythingOK := WithoutTOC
 else EverythingOK := WithTOC;
  case EverythingOK of
   True  : Form1.StatusBar1.Panels[0].Text := 'Done without errors!';
   False : Form1.StatusBar1.Panels[0].Text := 'Something went wrong :(';
  end;
  Form1.GenerateButton.Enabled := True;
end;
//===============================================

end.
